package com.macro.mall.tiny.modules.business.controller;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.google.common.collect.Maps;
import com.macro.mall.tiny.common.util.EprdmsHttpUtils;
import com.macro.mall.tiny.common.util.SpringUtils;
import com.macro.mall.tiny.config.CustomSpringConfigurator;
import com.macro.mall.tiny.modules.business.dto.SensorListenParam;
import com.macro.mall.tiny.modules.business.service.RedisSubscribeListener;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.listener.ChannelTopic;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.server.standard.SpringConfigurator;

import javax.websocket.*;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;

@Component
@ServerEndpoint(value = "/sensor/websocket", configurator = CustomSpringConfigurator.class)
@Slf4j
public class WebSocketServer {

    /**
     * 因为@ServerEndpoint不支持注入，所以使用SpringUtils获取IOC实例
     */
    private final RedisMessageListenerContainer redisMessageListenerContainer = SpringUtils.getBean(RedisMessageListenerContainer.class);

    private static final String topic = "collect:sensor";

    //静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
    private static final AtomicInteger onlineCount = new AtomicInteger(0);

    private static final Map<String, Integer> sensorMacCountMap = new HashMap<>();

    private static final Map<Session, RedisSubscribeListener> sessionListenerMap = new HashMap<>();

    @Value("${keystar.eprdms.url}")
    private String eprdmsUrl;

    private static final String OPEN_LISTEN_URL = "/eprdms/meteorology/sensor/start/listen";
    private static final String CLOSE_LISTEN_URL = "/eprdms/meteorology/sensor/stop/listen";
    private static final String GET_LISTEN_URL = "/eprdms/meteorology/sensor/current/listen";

    /**
     * 连接建立成功调用的方法
     *
     * @param session 可选的参数。session为与某个客户端的连接会话，需要通过它来给客户端发送数据
     */

    @OnOpen
    public void onOpen(Session session) {
        addOnlineCount();           //在线数加1
        log.info("websocket有新连接加入！当前在线人数为" + getOnlineCount());
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose(Session session) {
        subOnlineCount();           //在线数减1
        RedisSubscribeListener subscribeListener = sessionListenerMap.get(session);
        if (subscribeListener != null) {
            redisMessageListenerContainer.removeMessageListener(subscribeListener);
            Set<String> sensorMacSet = subscribeListener.getSensorMacSet();
            removeSensorMac(sensorMacSet);
        }
        if (WebSocketServer.onlineCount.get() == 0) {
            initSensorMac();
        }
        log.info("websocket有一连接关闭！当前在线人数为" + getOnlineCount());
    }


    /**
     * 收到客户端消息后调用的方法
     *
     * @param message 客户端发送过来的消息
     * @param session 可选的参数
     */
    @OnMessage
    public void onMessage(String message, Session session) {
        log.debug("websocket来自客户端的消息:" + message);
        RedisSubscribeListener subscribeListener = sessionListenerMap.get(session);
        if (subscribeListener == null) {
            subscribeListener = new RedisSubscribeListener();
            subscribeListener.setSession(session);
            //设置订阅topic
            redisMessageListenerContainer.addMessageListener(subscribeListener, new ChannelTopic(topic));
            sessionListenerMap.put(session, subscribeListener);
        }
        Set<String> sensorMacSet = subscribeListener.getSensorMacSet();
        removeSensorMac(sensorMacSet);
        sensorMacSet.addAll(Arrays.asList(message.split(",")));
        addSensorMac(sensorMacSet);
    }


    /**
     * 发生错误时调用
     *
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        log.error("websocket发生错误:", error);
    }



    public int getOnlineCount() {
        return onlineCount.get();
    }


    public void addOnlineCount() {
        WebSocketServer.onlineCount.getAndIncrement();
    }


    public void subOnlineCount() {
        WebSocketServer.onlineCount.getAndDecrement();
    }

    private void addSensorMac(Set<String> sensorMacSet) {
        List<String> openMacList = new ArrayList<>();
        sensorMacSet.forEach(sensorMac -> {
            Integer count = sensorMacCountMap.getOrDefault(sensorMac, 0);
            count++;
            if (count == 1) {
                openMacList.add(sensorMac);
            }
            sensorMacCountMap.put(sensorMac, count);
        });
        if (openMacList.isEmpty()) return;
        SensorListenParam sensorListenParam = new SensorListenParam(openMacList);
        JSONObject paramJsonObject = JSONUtil.parseObj(sensorListenParam);
        JSONArray resultArray = EprdmsHttpUtils.PostForEprdms(eprdmsUrl + OPEN_LISTEN_URL, paramJsonObject, JSONArray.class);
        if (resultArray == null) return;
        StringBuilder logSb = new StringBuilder("开启监听:").append("\r\n");
        resultArray.jsonIter().forEach(jsonObject -> {
            logSb.append("deviceMac:")
                    .append(jsonObject.getStr("deviceMac"))
                    .append(",result:")
                    .append(jsonObject.getStr("result"))
                    .append("\r\n");
        });
        log.info(logSb.toString());
    }

    private void removeSensorMac(Set<String> sensorMacSet) {
        if (sensorMacSet.isEmpty()) {
            return;
        }
        List<String> closeMacList = new ArrayList<>();
        sensorMacSet.forEach(sensorMac -> {
            Integer count = sensorMacCountMap.get(sensorMac);
            if (count == null) return;
            count--;
            if (count < 1) {
                sensorMacCountMap.remove(sensorMac);
                closeMacList.add(sensorMac);
            } else {
                sensorMacCountMap.put(sensorMac, count);
            }
        });
        sensorMacSet.clear();
        if (!closeMacList.isEmpty()) {
            closeListen(closeMacList);
        }
    }

    private void initSensorMac() {
        HashMap<String, String> paramMap = new HashMap<>();
        paramMap.put("monitorClient", "MONITOR_CLIENT_COLLECT");
        JSONObject result = EprdmsHttpUtils.getForEprdms(eprdmsUrl + GET_LISTEN_URL + "?monitorClient={monitorClient}", paramMap, JSONObject.class);
        if (result == null) return;
        JSONArray listingMacList = result.getJSONArray("sensorMacs");
        if (!(listingMacList == null || listingMacList.isEmpty())) {
            closeListen(listingMacList.toList(String.class));
        }
        sensorMacCountMap.clear();
        redisMessageListenerContainer.setMessageListeners(Maps.newHashMap());
    }

    private void closeListen(List<String> macList) {
        SensorListenParam sensorListenParam = new SensorListenParam(macList);
        JSONObject paramJsonObject = JSONUtil.parseObj(sensorListenParam);
        JSONArray resultArray = EprdmsHttpUtils.PostForEprdms(eprdmsUrl + CLOSE_LISTEN_URL, paramJsonObject, JSONArray.class);
        if (resultArray == null) return;
        StringBuilder logSb = new StringBuilder("关闭监听:").append("\r\n");
        resultArray.jsonIter().forEach(jsonObject -> {
            logSb.append("deviceMac:")
                    .append(jsonObject.getStr("deviceMac"))
                    .append(",result:")
                    .append(jsonObject.getStr("result"))
                    .append("\r\n");
        });
        log.info(logSb.toString());
    }

}