package com.macro.mall.tiny.modules.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.keystar.eprdms.core.utils.ShiroContextHolder;
import com.macro.mall.tiny.common.exception.Asserts;
import com.macro.mall.tiny.common.util.MyFileUtils;
import com.macro.mall.tiny.config.UploadConfig;
import com.macro.mall.tiny.modules.business.enums.DirectoryEnum;
import com.macro.mall.tiny.modules.business.mapper.KrBLineMapper;
import com.macro.mall.tiny.modules.business.mapper.KrBTowerMapper;
import com.macro.mall.tiny.modules.business.model.*;
import com.macro.mall.tiny.modules.business.service.BDirectoryCacheService;
import com.macro.mall.tiny.modules.business.service.FileService;
import com.macro.mall.tiny.modules.business.service.KrBLineCacheService;
import com.macro.mall.tiny.modules.business.service.KrBTowerService;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.nio.file.*;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.List;

import static com.macro.mall.tiny.common.util.UploadUtils.*;


/**
 * 文件上传服务
 */
@Service
@Slf4j
@EnableAsync
public class FileServiceImpl implements FileService {

    @Autowired
    private AsyncHandler asyncHandler;

    @Autowired
    private BDirectoryCacheService directoryCacheService;

    @Autowired
    private KrBTowerService towerService;

    @Autowired
    private KrBTowerMapper towerMapper;

    @Autowired
    private KrBLineMapper lineMapper;

    @Autowired
    private KrBLineCacheService lineCacheService;

    /**
     * 上传文件
     *
     * @param file
     */
    public void upload(MultipartFile file) throws IOException {
        String path = UploadConfig.path + file.getOriginalFilename();
        MyFileUtils.write(path, file.getInputStream());
    }

    /**
     * 分块上传文件
     *
     * @param size
     * @param chunks
     * @param chunk
     * @param file
     * @throws IOException
     */
    public void uploadWithBlock(String name,
                                String lineName,
                                String provinceId,
                                String cityId,
                                String voltage,
                                String lineStatus,
                                Long serviceBegintimeTs,
                                Integer beginTower,
                                Integer endTower,
                                Long size,
                                Integer chunks,
                                Integer chunk,
                                MultipartFile file,
                                String charset) throws IOException {
        putChunkMap(name, chunks);
        String target = UploadConfig.tempPath + name;
        MyFileUtils.writeWithBlock(target, size, file.getInputStream(), file.getSize(), chunks, chunk);
        addChunk(name, chunk);
        if (isUploaded(name)) {
            removeKey(name);
            String username = ShiroContextHolder.getInstance().getUsername();
            Subject subject = SecurityUtils.getSubject();
            asyncHandler.handleDocument(provinceId, cityId, target, lineName, voltage, lineStatus, serviceBegintimeTs, beginTower, endTower, username, subject, charset);
        }
    }

    @Override
    public BDirectory getRootDirectory() {
        BDirectory directory = directoryCacheService.getDirectory();
        if (directory != null) {
            return directory;
        }
        directory = MyFileUtils.getRootDirectory();
        if (directory != null) {
            directoryCacheService.setDirectory(directory);
        } else {
            directoryCacheService.setDirectory(new BDirectory());
        }
        return directory;
    }

    @Override
    public List<BFileDetail> getFileDetailList(String path) {
        return MyFileUtils.getFileDetailList(path);
    }

    @Override
    public List<BFile> getFileList(String path) {
        return MyFileUtils.getSubBFiles(path);
    }

    @Override
    public boolean deleteFile(String path) throws IOException {
//        boolean delete = ShiroAuthUtil.checkAccess(ShiroAuthUtil.EDIT);
//        if (!delete) {
//            Asserts.fail("您没有删除权限");
//        }
        Path filePath = Paths.get(path);
        if (!Files.exists(filePath)) {
            Asserts.fail("该文件不存在");
        }
        int powerPathCount = UploadConfig.getPowerPathCount();
        int relativeCount = filePath.getNameCount() - powerPathCount;
        if (relativeCount < 2) {
            return true;
        }
        if (Files.isDirectory(filePath)) {
            Files.walkFileTree(filePath, new SimpleFileVisitor<Path>() {
                @Override
                public FileVisitResult visitFile(Path file, BasicFileAttributes attributes) throws IOException {
                    Files.delete(file); // this will work because it's always a File
                    return FileVisitResult.CONTINUE;
                }

                @Override
                public FileVisitResult postVisitDirectory(Path dir, IOException exc) throws IOException {
                    Files.delete(dir); //this will work because Files in the directory are already deleted
                    return FileVisitResult.CONTINUE;
                }
            });
        } else {
            Files.delete(filePath);
        }

        if (relativeCount > 2) {
            String dirType = filePath.getName(powerPathCount + 2).toString();
            if (!dirType.equals(DirectoryEnum.TOWER_PIC.getName()) && !dirType.equals(DirectoryEnum.EARTH_WIRE_HARD_PIC.getName()) && !dirType.equals(DirectoryEnum.TEAM_CHECK_PIC.getName())) {
                return true;
            }
        }

        String provinceStr = filePath.getName(powerPathCount).toString();
        String lineNameStr = filePath.getName(powerPathCount + 1).toString();
        QueryWrapper<KrBLine> lineQueryWrapper = new QueryWrapper<>();
        lineQueryWrapper.lambda().eq(KrBLine::getProvince, provinceStr).eq(KrBLine::getLineName, lineNameStr);
        KrBLine line = lineMapper.selectOne(lineQueryWrapper);

        if (line != null) {
            QueryWrapper<KrBTower> towerQueryWrapper = new QueryWrapper<>();
            towerQueryWrapper.lambda().eq(KrBTower::getLineId, line.getId());

            if (relativeCount == 2) {
                lineMapper.deleteById(line.getId());
                towerMapper.delete(towerQueryWrapper);
            } else {
                List<KrBTower> towerList = towerMapper.selectList(towerQueryWrapper);
                towerService.setHasFile(towerList);
            }
        }

        // 后续清楚缓存处理
        lineCacheService.delLineList();
        directoryCacheService.delDirectory();
        return true;
    }

}
